@rem = '--*-Perl-*--
@echo off

rem Licensed Materials - Property of IBM
rem Rational ClearCase
rem (C) Copyright IBM Corp. 2001, 2007. All Rights Reserved.
rem US Government Users Restricted Rights
rem Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

rem cqcc_launch.bat: This file will launch the CQCC integration
rem after deciding whether to use ratlperl or cqperl (preferred)
rem and setting up include directories, etc. 
rem 
rem This file consists of a command header (between the @rem lines)
rem and an embedded perl script. It figures out how to launch
rem perl in order to run the embedded perl commands below.
rem Perl treats the header section as a list assignment and
rem Windows treats the @rem lines as remarks, using the lines in
rem between as commands.

set PATH_SEARCH=
if NOT EXIST %0 set PATH_SEARCH=-S

if "%OS%" == "Windows_NT" goto Windows
ratlperl -x %PATH_SEARCH% %0 %1 %2 %3 %4 %5 %6 %7 %8 %9
if errorlevel 1 exit %errorlevel%
goto endofperl

:Windows
ratlperl -x %PATH_SEARCH% %0 %*
if NOT "%COMSPEC%" == "%SystemRoot%\system32\cmd.exe" goto endofperl
if %errorlevel% == 9009 echo You do not have ratlperl in your PATH.
if errorlevel 1 goto script_failed_so_exit_with_non_zero_val 2>nul
goto endofperl
@rem ';

#!perl

use strict;
require Config;

if ($ENV{ATRIA_WEB_GUI} && ! defined $ENV{ATRIA_WEB_CLEARPROMPT}) {
   print "ERROR: The Base ClearCase/ClearQuest Integration is not currently \n" .
         "supported under ClearCase Web Servers\n";
   # Checkouts aren't stopped so allow users to uncheckout files without error
   if ($ENV{CLEARCASE_OP_KIND} eq "uncheckout") {
       exit 0;
   } 
   exit 2;
}

# Determine OS-specific variables for filename handling and executables

my $windowsOS = ($Config::Config{osname} eq "MSWin32") ? 1 : 0;
my ($cqperl, $perl, $SEP, $QUOTE, $PATH_SEP, $TEMP_DIR);
if ($windowsOS) {
    $perl = "ratlperl";
    $cqperl = "cqperl.EXE";
    $SEP = "\\";   # Should print as \
    $QUOTE = "\"";
    $PATH_SEP = ";";
    $TEMP_DIR = $ENV{'TMPDIR'} ? $ENV{'TMPDIR'} : 
                $ENV{'TMP'} ? $ENV{'TMP'} :
                $ENV{'TEMP'} ? $ENV{'TEMP'} :
                (-w "c:\\TEMP" ? "c:\\TEMP" : "");

    if ($TEMP_DIR eq "") {
        #display a failure message and terminate the process
        # Be consistant with main trigger
        print("Error: Please set TMPDIR or TMP or TEMP environment varible.\n");
        exit 2;
    }
} else {
    $perl = "/opt/rational/common/bin/ratlperl";
    $cqperl = "cqperl";
    $SEP = "/";
    $QUOTE = "";
    $PATH_SEP = ":";
    $TEMP_DIR = $ENV{'TMPDIR'} ? $ENV{'TMPDIR'} :
                (-w "/tmp" ? "/tmp" : ".");

    # If clearquest is installed but not setup
    # get the clearquest environment settings setup
    my $clearquest = "/opt/rational/clearquest/bin/clearquest";
    if (! defined $ENV{CQ_ARCH} && -x $clearquest) {
        eval `$clearquest -dumpperl`;
    }

    # If Clearcase home not in path, add it
    my $cchome = "/opt/rational/clearcase/bin";
    if ($ENV{PATH} !~ /$cchome/) {
        $ENV{PATH} .= ":$cchome";
    }
}

my $path;
my $centralDir = "";

# Get the pathname argument if it is provided. 

if (defined $ARGV[0] && substr($ARGV[0],0,1) ne "-") {
    $path = shift @ARGV;
}

# Filter out launch-script only arguments
# Avoids sensitivity to argument pair order

my @triggerArgs;
my $launchCmd="";

while (@ARGV) {
    my $arg = shift @ARGV;

    if ($arg eq "-central") {
       $centralDir = shift @ARGV;
       if (defined $path) {
           $path = $centralDir . $SEP . $path;
       } else {
           $path = $centralDir . $SEP . "config.pl";
       }
       if ($ENV{"CQCC_LAUNCH_DEBUG"} > 0) {
           print "\nReading config file and triggers from $centralDir\n";
       }

    # -vob flag tells script to get basic command line arguments
    # from the checkout trigger in the current vob which will provide 
    # the correct paths to cqcc_launch, the config file, and trigger source.

    } elsif ($arg eq "-vob") {
       my $trigger;
       my $vobname = shift @ARGV;
       if (defined $vobname && substr($vobname,0,1) ne "-") {
           $trigger = `cleartool describe trtype:cq_co_trigger\@$vobname`;
       } else {
           unshift @ARGV,$vobname;
           $trigger = `cleartool describe trtype:cq_co_trigger`;
       }
       if (defined $ENV{"CLEARCASE_OP_KIND"}) {
           print "The -vob option can not be used inside a trigger\n";
           exit 1;
       } elsif ($trigger =~ /cqcc_launch/) {
           my $plat = $windowsOS ? "win" : "unix";
           # The checkout trigger has all the necessary centralization args
           if ($trigger =~ /\-exec$plat (.*)/) {
               my $platTrigger = $1;
               if ($platTrigger =~ /cqcc_launch/) {
                   $launchCmd = $platTrigger;
               }
           }
           if ($launchCmd eq "") {
               print "Unable to find checkout trigger using cqcc_launch\n";
               print $trigger;
               exit 1;
           }
        }
    } else {
       push @triggerArgs, $arg;
    }
}

my $arglist = join(" ", @triggerArgs);
my $cqcc_incdir = "";
my $perl_switch = $ENV{CQCC_LAUNCH_DEBUG} > 1 ? "-d" : "";
my $temp;
my $cmd;

if ($launchCmd ne "") {
    $cmd = "$launchCmd $arglist";
} else {
    # Catch case where no path is provided. 
    if (! defined $path) {
        $path = $windowsOS ? "CQCC\\config.pl" : "CQCC/config.pl";
        if ($ENV{"CQCC_LAUNCH_DEBUG"} > 0) {
            print "No config file provided, assuming $path\n";
        }
    }

    # Use cqperl if found, unless CQCC_LAUNCH_RATLPERL prevents it
    if (! defined $ENV{CQCC_LAUNCH_RATLPERL} ||
            $ENV{CQCC_LAUNCH_RATLPERL} ne "1" ) {
        for $temp (split($PATH_SEP,$ENV{PATH})) {
            if (-x $temp . $SEP . $cqperl) {
                $perl=$cqperl;
                last;
            }
        } 
    }

    # Expect CQCC to reside in Clearcase\lib\CQCC by default.
    # Once found pass this directory on to cqperl it that is being used

    if ($centralDir ne "") {
        $cqcc_incdir = "-I$centralDir" . $SEP . "..";
    } else {
       for $temp (@INC) {
           # CQCC resides in old clearcase lib area - cover for RatlPerl @INC path
           if ($temp =~ /(.+)common(.+)lib(.+)perl5(.+)/i) { 
               $temp = "$1" . "clearcase" . $SEP . "lib"; 
           }   
           if (-e $temp . $SEP . "CQCC") {
               $cqcc_incdir = "-I$QUOTE$temp$QUOTE";
               if (substr($path,0,4) eq "CQCC") {
                  $path = $temp . $SEP . $path;
               }
              last;
           }
       }
    }

    $cmd= "$perl $perl_switch $cqcc_incdir " .
    "$QUOTE$path$QUOTE -path $QUOTE$path$QUOTE $arglist";
}

my $ret = 1;
if ($ENV{"CQCC_LAUNCH_DEBUG"} > 0) {
   print "\nCMD: $cmd\n\n";
}

# Work around CQPerl return code problem, can't use backticks
my $retFile = $TEMP_DIR . $SEP . "cqcc_launch.$ENV{CLEARCASE_PPID}";

unlink $retFile;

$ret = system $cmd;

# Note: The -vob option will not return an accurate exit status 
# beyond zero and non-zero(error).

if (-e $retFile) { 
    open(INFILE, $retFile);
    $ret = <INFILE>;
    close(INFILE);
    unlink $retFile;
    if ($ENV{"CQCC_LAUNCH_DEBUG"} > 0) {
       print "\nReading result from $retFile\n";
    }
} 

if ($ENV{"CQCC_LAUNCH_DEBUG"} > 0) {
   print "\nReturn: $ret\n";
}

exit $ret;

__END__
:endofperl
